FromTokens

Derive Macro FromTokens 

#[derive(FromTokens)]
Provides the FromTokenSink trait for a struct or enum. This is used to deserialize into Rust data structures.

The FromTokens trait has a blanket implementation that uses FromTokenSink.