PrintingTokenSink

Struct PrintingTokenSink 

pub struct PrintingTokenSink<'a: 'b, 'b, S: TokenSink, W: Write> { /* private fields */ }
A token sink that prints a description of the tokens to a writer, and forwards them to another sink. For debugging.

Each token is output on one line. Start and end tokens adjust indentation.

Example

let mut got = TokenVec::new();
let mut output = Vec::new();
vec![42u32]
    .into_tokens(&mut PrintingTokenSink::new(&mut got, &mut output, "T: "))
    .unwrap();
assert_eq!(
    String::from_utf8(output).unwrap(),
    concat!(
        "T: Seq(SeqMeta { size_hint: Some(1) })\n",
        "T:   U32(42)\n",
        "T: EndSeq\n")
);

Implementations

Source

impl<'a: 'b, 'b, S: TokenSink, W: Write> PrintingTokenSink<'a, 'b, S, W>

pub fn new(sink: &'b mut S, writer: W, prefix: &'a str) -> Self

Creates a new printing sink. The prefix is used for each output line, before the indentation.