pub enum TokenType {
Show 29 variants
Unit,
Bool,
U8,
U16,
U32,
U64,
U128,
Usize,
I8,
I16,
I32,
I64,
I128,
Isize,
F32,
F64,
Char,
Bytes,
Str,
Seq,
EndSeq,
Tuple,
EndTuple,
Struct,
Field,
EndStruct,
Enum,
Variant,
EndEnum,
}Expand description
The type of a token, excluding any value or metadata.
This is used by sinks to declare expected tokens.
Variants
Unit
Bool
U8
U16
U32
U64
U128
Usize
I8
I16
I32
I64
I128
Isize
F32
F64
Char
Bytes
Str
Seq
EndSeq
Tuple
EndTuple
Struct
Field
EndStruct
Enum
Variant
EndEnum