pub struct PrintingTokenSink<'a: 'b, 'b, S: TokenSink, W: Write> { /* private fields */ }Expand description
A token sink that prints a description of the tokens to a writer, and forwards them to another sink. For debugging.
Each token is output on one line. Start and end tokens adjust indentation.
Example
Implementations